@@ -25,7 +25,7 @@
  */
 
 #include "py/runtime.h"
-#include "extmod/machine_spi.h"
+#include "extmod/modmachine.h"
 #include "spi.h"
 
 /******************************************************************************/
